package com.vaadin.demo.tutorial.addressbook.ui;

import java.beans.BeanInfo;
import java.beans.IntrospectionException;
import java.beans.Introspector;
import java.beans.PropertyDescriptor;
import java.text.NumberFormat;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;
import java.util.List;
import java.util.Locale;

import com.vaadin.data.Item;
import com.vaadin.data.Validator;
import com.vaadin.data.fieldgroup.BeanFieldGroup;
import com.vaadin.data.util.BeanItem;
import com.vaadin.data.util.converter.StringToIntegerConverter;
import com.vaadin.data.validator.EmailValidator;
import com.vaadin.demo.tutorial.addressbook.AddressBookUI;
import com.vaadin.demo.tutorial.addressbook.data.Person;
import com.vaadin.demo.tutorial.addressbook.data.PersonContainer;
import com.vaadin.server.Sizeable.Unit;
import com.vaadin.ui.Button;
import com.vaadin.ui.Button.ClickEvent;
import com.vaadin.ui.Button.ClickListener;
import com.vaadin.ui.ComboBox;
import com.vaadin.ui.Field;
import com.vaadin.ui.HorizontalLayout;
import com.vaadin.ui.Notification;
import com.vaadin.ui.Notification.Type;
import com.vaadin.ui.TextField;

@SuppressWarnings("serial")
public class PersonFieldGroup extends BeanFieldGroup<Person> implements ClickListener {

  private Button save = new Button("Save", (ClickListener) this);
  private Button cancel = new Button("Cancel", (ClickListener) this);
  private Button edit = new Button("Edit", (ClickListener) this);
  private final ComboBox cities = new ComboBox("City");

  private AddressBookUI app;
  private boolean newContactMode = false;
  private Person newPerson = null;
  private HorizontalLayout footer = new HorizontalLayout();

  public PersonFieldGroup(AddressBookUI app) {
    super(Person.class);
    this.app = app;

    /*
     * Enable buffering so that commit() must be called for the form before
     * input is written to the data. (Form input is not written immediately
     * through to the underlying object.)
     */
    setBuffered(true);

    footer.setSpacing(true);
    footer.addComponent(save);
    footer.addComponent(cancel);
    footer.addComponent(edit);
    footer.setVisible(false);

    /* Allow the user to enter new cities */
    cities.setNewItemsAllowed(true);
    /* We do not want to use null values */
    cities.setNullSelectionAllowed(false);
    /* Add an empty city used for selecting no city */
    cities.addItem("");
    cities.setWidth(200, Unit.PIXELS);

    /* Populate cities select using the cities in the data container */
    PersonContainer ds = app.getDataSource();
    for (Iterator<Person> it = ds.getItemIds().iterator(); it.hasNext();) {
      String city = (it.next()).getCity();
      cities.addItem(city);
    }
  }

  @Override
  @SuppressWarnings({ "rawtypes", "unchecked" })
  protected <T extends Field> T build(String caption, Class<?> dataType, Class<T> fieldType) throws BindException {
    if (caption.equals("City")) {
      return (T) cities;
    }
    T field = super.build(caption, dataType, fieldType);
    field.setWidth(200, Unit.PIXELS);
    if (caption.equals("Postal Code")) {
      TextField tf = (TextField) field;
      tf.setNullRepresentation("");
      tf.setConverter(new StringToIntegerConverter() {
        @Override
        protected NumberFormat getFormat(Locale locale) {
          NumberFormat format = NumberFormat.getIntegerInstance(app.getLocale());
          format.setGroupingUsed(false);
          return format;
        }
      });
      tf.addValidator(new Validator() {

        @Override
        public void validate(Object value) throws InvalidValueException {
          Integer i = (Integer) value;
          if (10000 <= i && i < 100000) {
            return;
          }
          throw new InvalidValueException("Postal code must be 5 digits long and start with non-zero digit.");
        }

      });
      tf.setRequired(true);
    }
    if (caption.equals("Email")) {
      field.addValidator(new EmailValidator("Email must contain '@' and have full domain."));
      field.setRequired(true);
    }
    return field;
  }

  public void buttonClick(ClickEvent event) {
    Button source = event.getButton();

    if (source == save) {
      /* If the given input is not valid there is no point in continuing */
      if (!isValid()) {
        return;
      }

      try {
        commit();
      } catch (CommitException e) {
        Notification.show("Commit failed! " + e.getMessage(), Type.ERROR_MESSAGE);
      }
      if (newContactMode) {
        /* We need to add the new person to the container */
        Item addedItem = app.getDataSource().addItem(newPerson);
        /*
         * We must update the form to use the Item from our datasource as we are
         * now in edit mode (no longer in add mode)
         */
        setItemDataSource(addedItem);

        newContactMode = false;
      }
      setReadOnly(true);
    } else if (source == cancel) {
      if (newContactMode) {
        newContactMode = false;
        /* Clear the form and make it invisible */
        setItemDataSource((Item) null);
      } else {
        discard();
      }
      setReadOnly(true);
    } else if (source == edit) {
      setReadOnly(false);
    }
  }

  public HorizontalLayout getFooter() {
    return footer;
  }

  @Override
  public void setItemDataSource(Item newDataSource) {
    newContactMode = false;

    if (newDataSource != null) {
      super.setItemDataSource(newDataSource);

      setReadOnly(true);
      getFooter().setVisible(true);
    } else {
      super.setItemDataSource((Person) null);
      getFooter().setVisible(false);
    }
  }

  @Override
  public void setReadOnly(boolean readOnly) {
    super.setReadOnly(readOnly);
    save.setVisible(!readOnly);
    cancel.setVisible(!readOnly);
    edit.setVisible(readOnly);
  }

  public void addContact() {
    // Create a temporary item for the form
    newPerson = new Person();
    setItemDataSource(new BeanItem<Person>(newPerson));
    newContactMode = true;
    setReadOnly(false);
  }

  @Override
  public Collection<Object> getUnboundPropertyIds() {
    if (getItemDataSource() != null) {
      return super.getUnboundPropertyIds();
    }
    List<Object> unboundPropertyIds = new ArrayList<Object>();
    try {
      BeanInfo info = Introspector.getBeanInfo(Person.class);
      for (PropertyDescriptor d : info.getPropertyDescriptors()) {
        if (d.getReadMethod().getDeclaringClass() != info.getBeanDescriptor().getBeanClass()) {
          continue;
        }
        unboundPropertyIds.add(d.getName());
      }
    } catch (IntrospectionException e) {
      throw new Error("Couldn't create bean information while looking for properties.", e);
    }

    unboundPropertyIds.removeAll(getBoundPropertyIds());
    return unboundPropertyIds;
  }

}